/*
 . Live Coding @ Atelier hyper media, Aix-en-provence
 . 16-19 Decembre 2008
 .
 . Use with Processing 1.0.1
 .
 . Julien 'v3ga' Gachadoat
 . http://v3ga.net/
 */

// ========================================================
// Server config
// ========================================================
// Font
PFont font,font2;

// Timer
timer t;
float tAcc = 0.0;

// Server
//String myRemoteHost = "10.0.1.3"; // change, depends on your network ip & server ip
String myRemoteHost = "192.168.2.2"; // change, depends on your network ip & server ip
String myUserName = "v333ga";
int myRemotePort = 7400;
int myListeningPort = 7401;
boolean isConnect = false;

// Protocol for live coding
String verbChangeSceneScript = "/vf/livecoding/script/change";
String verbConnect = "/vf/livecoding/connect";
String verbDisconnect = "/vf/livecoding/disconnect";
String verbMouseMoved = "/vf/livecoding/mousemoved";
String verbMousePressed = "/vf/livecoding/mousepressed";


boolean isCallInit = true;

// Period sending script
float tSendScript = 1.0;
String lastReceivedCommand = "";
String lastReceivedMessage = "";
boolean stopSending = false;
boolean isSendScript = true;

// Editor
String pathEditor = "/Applications/TextEdit.app"; // different on each OS (see "open" documentation)
String pathScript = null;
String nameScript = "";

// Virtual Screen
VirtualScreen virtualScreen;


// ========================================================
// import
// ========================================================
import oscP5.*;
import netP5.*;

// ========================================================
// setup()
// ========================================================
void setup()
{
  // Window
  size(500,500);
  frameRate(30);
  smooth();
  
  // Fonts
  font = loadFont("Arial-Black-30.vlw");
  font2 = loadFont("ArialMT-12.vlw");

  // Virtual Screen
  virtualScreen = new VirtualScreen(10,150,width-20,height-10-150);

  // Init
  initTimer();
  initOsc(); 

  // Send 'connect' message
  //sendConnect();
}

// ========================================================
// draw()
// ========================================================
void draw()
{
  background(0);

  sendScript();
  drawVirtualScreen();
  drawInfos();
}


// ========================================================
// sendScript()
// ========================================================
void sendScript()
{
  tAcc += t.M_getDelta() / 1000.0;
  if (tAcc >= tSendScript)
  {
    tAcc = 0.0;
    if (isSendScript)
    {
      if (sendNewScript())
        background(255,0,0);
    }
  }
}

// ========================================================
// drawVirtualScreen()
// ========================================================
void drawVirtualScreen()
{
  virtualScreen.draw();
}

// ========================================================
// drawInfos()
// ========================================================
void drawInfos()
{
  fill(255);

  // Title
  textFont(font, 16);
  text("Live coding client", 5, 15);

  // Serveur infos
  textFont(font2, 12);
  text("+ Server : "+myRemoteHost+" (port "+myRemotePort+")" + " - " + (isConnect ? "connected" : "*not* connected"), 5, 30);

  // Script loaded
  text("+ Script : "+(nameScript.equals("") ? "???" : nameScript), 5, 45);

  if (isCallInit)
    text("+ Calling init() on next reload !", 5, 60);
    
  
  // Infos from server
if (true)
{
  text("+ Server said: "+(lastReceivedCommand.equals("") ? "[none...]" : lastReceivedCommand), 5, 75);
  text("+ Server message: "+(lastReceivedMessage.equals("") ? "[silence...]" : lastReceivedMessage), 5, 90);
}
}

// ========================================================
// openEditor()
// ========================================================
void openEditor()
{
  String newPathScript = selectInput();
  if (newPathScript == null) 
  {
      println("No file was selected...");
  } 
  else 
  {
    pathScript = newPathScript;
    String[] params = {pathEditor, pathScript};
    open(params);
    
    // get the name of the script
    println("- loading "+pathScript);
    String[] tmp = split(pathScript, '/');
    if (tmp.length>0)
      nameScript = tmp[tmp.length-1];
    
    // New script -> call init !    
    isCallInit = true;
  }
}

// ========================================================
// keyPressed()
// ========================================================
void keyPressed()
{
  if (key == 'r' || key == 'R')
     isCallInit = true;
  if (key == 'c' || key == 'C')
    sendConnect();    
  if (key == 'd' || key == 'D')
    sendDisconnect();    
  if (key == 'o' || key == 'O')
    openEditor();    
}

// ========================================================
// stop()
// not called on cmd + Q ?
// ========================================================
void stop()
{
  println("- stop");
  sendDisconnect();
  super.stop();
}

// ========================================================
// void mouseMoved
// void mousePressed
// ========================================================
void mouseMoved()
{
  virtualScreen.mouseMoved();
}

void mousePressed()
{
  virtualScreen.mousePressed();
}

